##############################################################################
#
# Copyright (c) 2001 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" CMFDefault portal_syndication tool.

Manage outbound RSS syndication of folder content.
"""

from datetime import datetime
from warnings import warn

from AccessControl.SecurityInfo import ClassSecurityInfo
from Acquisition import aq_base
from App.class_init import InitializeClass
from App.special_dtml import HTMLFile
from DateTime.DateTime import DateTime
from OFS.SimpleItem import SimpleItem

from zope.component import getAdapter, queryAdapter
from zope.interface import implements

from Products.CMFCore.interfaces import ISyndicationTool, ISyndicationInfo
from Products.CMFCore.interfaces import IFolderish
from Products.CMFCore.PortalFolder import PortalFolderBase
from Products.CMFCore.utils import _checkPermission
from Products.CMFCore.utils import registerToolInterface
from Products.CMFCore.utils import UniqueObject
from Products.CMFDefault.exceptions import AccessControl_Unauthorized
from Products.CMFDefault.permissions import ManagePortal
from Products.CMFDefault.permissions import ManageProperties
from Products.CMFDefault.utils import _dtmldir


class SyndicationError(Exception):

    pass


class SyndicationTool(UniqueObject, SimpleItem):
    """ The syndication tool manages the site-wide policy for
        syndication of folder content as RSS.
    """
    implements(ISyndicationTool)

    id = 'portal_syndication'
    meta_type = 'Default Syndication Tool'

    security = ClassSecurityInfo()

    #Default Sitewide Values
    isAllowed = enabled = False
    syUpdatePeriod = period = 'daily'
    syUpdateFrequency = frequency = 1
    syUpdateBase = base = datetime.now()
    max_items = 15

    #ZMI Methods
    manage_options = ( ( { 'label'  : 'Overview'
                         , 'action' : 'overview'
                         , 'help'   : ( 'CMFDefault'
                                      , 'Syndication-Tool_Overview.stx' )
                         },
                        )
                     )

    security.declareProtected(ManagePortal, 'overview')
    overview = HTMLFile('synOverview', _dtmldir)

    def _syndication_info(self, obj):
        """Get a SyndicationInfo adapter for managing object
        syndication settings
        """
        adapter = queryAdapter(obj, ISyndicationInfo)
        if adapter is None:
            raise SyndicationError("Syndication is not possible")
        return adapter

    security.declareProtected(ManageProperties, 'editSyInformationProperties')
    def editSyInformationProperties( self
                                   , obj
                                   , updatePeriod=None
                                   , updateFrequency=None
                                   , updateBase=None
                                   , max_items=None
                                   , REQUEST=None
                                   ):
        """
        Edit syndication properties for the obj being passed in.
        These are held on the syndication_information object.
        Not Sitewide Properties.
        """
        info = self.getSyndicationInfo(obj)

        if not info.enabled:
            raise SyndicationError('Syndication is Disabled')

        info.period = updatePeriod or self.period
        info.frequency = updateFrequency or self.frequency
        info.base = updateBase or self.base
        info.max_items = max_items or self.max_items

    security.declareProtected(ManageProperties, 'enableSyndication')
    def enableSyndication(self, obj):
        """
        Enable syndication for the obj
        """
        if not self.isSiteSyndicationAllowed():
            raise SyndicationError('Syndication is Disabled')

        info = self._syndication_info(obj)
        info.enable()

    security.declareProtected(ManageProperties, 'disableSyndication')
    def disableSyndication(self, obj):
        """
        Disable syndication for the obj; and remove it.
        """
        info = self._syndication_info(obj)
        info.disable()

    security.declarePublic('getSyndicatableContent')
    def getSyndicatableContent(self, obj):
        """
        An interface for allowing folderish items to implement an
        equivalent of PortalFolderBase.contentValues()
        """
        if hasattr(obj, 'synContentValues'):
            values = obj.synContentValues()
        else:
            values = PortalFolderBase.contentValues(obj)
            values = [v for v in values if not IFolderish.providedBy(v)]
        return values

    security.declarePublic('buildUpdatePeriods')
    def buildUpdatePeriods(self):
        """
        Return a list of possible update periods for the xmlns: sy
        """
        updatePeriods = ( ('hourly',  'Hourly')
                        , ('daily',   'Daily')
                        , ('weekly',  'Weekly')
                        , ('monthly', 'Monthly')
                        , ('yearly',  'Yearly')
                        )
        return updatePeriods

    security.declarePublic('isSiteSyndicationAllowed')
    def isSiteSyndicationAllowed(self):
        """
        Return sitewide syndication policy
        """
        return self.enabled

    security.declarePublic('isSyndicationAllowed')
    def isSyndicationAllowed(self, obj=None):
        """
        Check whether syndication is enabled for the site.  This
        provides for extending the method to check for whether a
        particular obj is enabled, allowing for turning on only
        specific folders for syndication.
        """
        if obj is None:
            return self.isSiteSyndicationAllowed()
        try:
            info = self._syndication_info(obj)
        except SyndicationError:
            return False
        return getattr(info, 'enabled', False)

    security.declarePublic('getSyndicationInfo')
    def getSyndicationInfo(self, obj):
        """
        Return a dictionary of syndication parameters
        """
        info = self._syndication_info(obj)
        if not info.enabled:
            raise SyndicationError('Syndication is not allowed')
        return info

    security.declarePublic('getUpdatePeriod')
    def getUpdatePeriod( self, obj=None ):
        """
        Return the update period for the RSS syn namespace.
        This is either on the object being passed or the
        portal_syndication tool (if a sitewide value or default
        is set)

        NOTE:  Need to add checks for sitewide policies!!!
        """
        if obj is not None:
            period = self.getSyndicationInfo(obj).period
        else:
            period = self.period
        return period

    security.declarePublic('getUpdateFrequency')
    def getUpdateFrequency(self, obj=None):
        """
        Return the update frequency (as a positive integer) for
        the syn namespace.  This is either on the object being
        pass or the portal_syndication tool (if a sitewide value
        or default is set).

        Note:  Need to add checks for sitewide policies!!!
        """
        if obj is not None:
            frequency = self.getSyndicationInfo(obj).frequency
        else:
            frequency = self.frequency

        return frequency

    security.declarePublic('getUpdateBase')
    def getUpdateBase(self, obj=None):
        """
        Return the base date to be used with the update frequency
        and the update period to calculate a publishing schedule.

        Note:  I'm not sure what's best here, creation date, last
        modified date (of the folder being syndicated) or some
        arbitrary date.  For now, I'm going to build a updateBase
        time from zopetime and reformat it to meet the W3CDTF.
        Additionally, sitewide policy checks might have a place
        here...
        """
        if obj is not None:
            base = self.getSyndicationInfo(obj).base
        else:
            base = self.base
        return base.isoformat()

    security.declarePublic('getHTML4UpdateBase')
    def getHTML4UpdateBase(self, obj=None):
        """
        Return HTML4 formated UpdateBase DateTime
        """
        warn("RSS 2.0 uses RFC 822 formatting"
             " this method will be removed in CMF 2.4",
             DeprecationWarning, stacklevel=2)
        # HTML4 is almost the same as isoformat()
        if obj is not None:
            base = obj.base
        else:
            base = self.base
        # normalise
        base = base.replace(tzinfo=None)
        as_zope = DateTime(base.isoformat())
        return as_zope.HTML4()

    def getMaxItems(self, obj=None):
        """
        Return the max_items to be displayed in the syndication
        """
        if obj is not None:
            max_items = self.getSyndicationInfo(obj).max_items
        else:
            max_items = self.max_items
        return max_items

InitializeClass(SyndicationTool)
registerToolInterface('portal_syndication', ISyndicationTool)



##parameters=member=None, password='secret', email='foo@example.org'
##
from Products.CMFCore.utils import getUtilityByInterfaceName
from Products.CMFDefault.utils import decode
from Products.CMFDefault.utils import makeEmail
from Products.CMFDefault.utils import Message as _

atool = getUtilityByInterfaceName('Products.CMFCore.interfaces.IActionsTool')
ptool = getUtilityByInterfaceName('Products.CMFCore.interfaces.IPropertiesTool')
utool = getUtilityByInterfaceName('Products.CMFCore.interfaces.IURLTool')
portal_url = utool()


options = {}

options['portal_title'] = ptool.title()
options['portal_description'] = ptool.getProperty('description')
options['portal_url'] = portal_url

member_id = member and member.getId() or 'foo'
options['member_id'] = member_id
options['password'] = password

target = atool.getActionInfo('user/login')['url']
options['login_url'] = '%s' % target

email_from_name = ptool.getProperty('email_from_name')
options['signature'] = email_from_name

headers = {}
headers['Subject'] = _(u'${portal_title}: Your Membership Information',
                      mapping={'portal_title': decode(ptool.title(), script)})
headers['From'] = '%s <%s>' % (email_from_name,
                               ptool.getProperty('email_from_address'))
headers['To'] = '<%s>' % email

mtext = context.registered_email_template(**decode(options, script))
return makeEmail(mtext, script, headers)



##############################################################################
#
# Copyright (c) 2002 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Unit tests for utils module.
"""

import unittest



class DefaultUtilsTests(unittest.TestCase):

    COMMON_HEADERS = '''Author: Tres Seaver
Title: Test Products.CMFDefault.utils.parseHeadersBody'''

    MULTILINE_DESCRIPTION = '''Description: this description spans
        multiple lines.'''

    MULTIPARAGRAPH_DESCRIPTION = \
        '''Description: this description spans multiple lines.
        
        It includes a second paragraph'''

    TEST_BODY = '''Body goes here, and can span multiple
lines.  It can even include "headerish" lines, like:

Header: value
'''

    def setUp(self):
        from zope.component.testing import setUp
        setUp()

    def tearDown(self):
        from zope.component.testing import tearDown
        tearDown()

    def test_parseHeadersBody_no_body( self ):
        from Products.CMFDefault.utils import parseHeadersBody

        headers, body = parseHeadersBody( '%s\n\n' % self.COMMON_HEADERS )
        self.assertEqual( len( headers ), 2 )
        self.failUnless( 'Author' in headers.keys() )
        self.assertEqual( headers[ 'Author' ], 'Tres Seaver' )
        self.failUnless( 'Title' in headers.keys() )
        self.assertEqual( len( body ), 0 )

    def test_parseHeadersBody_continuation( self ):
        from Products.CMFDefault.utils import parseHeadersBody

        headers, body = parseHeadersBody( '%s\n%s\n\n'
                                        % ( self.COMMON_HEADERS
                                          , self.MULTILINE_DESCRIPTION
                                          )
                                        )
        self.assertEqual( len( headers ), 3 )
        self.failUnless( 'Description' in headers.keys() )
        desc_len = len( headers[ 'Description' ].split('\n') )
        self.assertEqual( desc_len, 2 )
        self.assertEqual( len( body ), 0 )

    def test_parseHeadersBody_embedded_blank_line( self ):
        from Products.CMFDefault.utils import parseHeadersBody

        headers, body = parseHeadersBody( '%s\n%s\n\n%s'
                                        % ( self.COMMON_HEADERS
                                          , self.MULTIPARAGRAPH_DESCRIPTION
                                          , self.TEST_BODY
                                          )
                                        )
        self.assertEqual( len( headers ), 3 )
        self.failUnless( 'Description' in headers.keys() )
        desc_lines = headers[ 'Description' ].split('\n')
        desc_len = len( desc_lines )
        self.assertEqual( desc_len, 3, desc_lines )
        self.assertEqual( desc_lines[1], ' ' )
        self.assertEqual( body, self.TEST_BODY )

    def test_parseHeadersBody_body( self ):
        from Products.CMFDefault.utils import parseHeadersBody

        headers, body = parseHeadersBody( '%s\n\n%s'
                                        % ( self.COMMON_HEADERS
                                          , self.TEST_BODY
                                          )
                                        )
        self.assertEqual( len( headers ), 2 )
        self.assertEqual( body, self.TEST_BODY )

    def test_parseHeadersBody_body_malformed_terminator( self ):
        from Products.CMFDefault.utils import parseHeadersBody

        headers, body = parseHeadersBody( '%s\n \n%s'
                                        % ( self.COMMON_HEADERS
                                          , self.TEST_BODY
                                          )
                                        )
        self.assertEqual( len( headers ), 2 )
        self.assertEqual( body, self.TEST_BODY )

    def test_parseHeadersBody_preload( self ):
        from Products.CMFDefault.utils import parseHeadersBody

        preloaded = { 'Author' : 'xxx', 'text_format' : 'structured_text' }
        headers, body = parseHeadersBody( '%s\n%s\n\n%s'
                                        % ( self.COMMON_HEADERS
                                          , self.MULTILINE_DESCRIPTION
                                          , self.TEST_BODY
                                          )
                                        , preloaded
                                        )
        self.assertEqual( len( headers ), 4 )
        self.assertNotEqual( preloaded[ 'Author' ], headers[ 'Author' ] )
        self.assertEqual( preloaded[ 'text_format' ], headers[ 'text_format' ] )

    def test_scrubHTML_no_adapter_falls_back(self):
        from Products.CMFDefault.utils import scrubHTML

        self.assertEqual( scrubHTML('<a href="foo.html">bar</a>'),
                          '<a href="foo.html">bar</a>' )
        self.assertEqual( scrubHTML('<b>bar</b>'),
                          '<b>bar</b>' )
        self.assertEqual( scrubHTML('<base href="" /><base>'),
                          '<base href="" /><base />' )
        self.assertEqual( scrubHTML('<blockquote>bar</blockquote>'),
                          '<blockquote>bar</blockquote>' )
        self.assertEqual( scrubHTML('<body bgcolor="#ffffff">bar</body>'),
                          '<body bgcolor="#ffffff">bar</body>' )
        self.assertEqual( scrubHTML('<br /><br>'),
                          '<br /><br />' )
        self.assertEqual( scrubHTML('<hr /><hr>'),
                          '<hr /><hr />' )
        self.assertEqual( scrubHTML('<img src="foo.png" /><img>'),
                          '<img src="foo.png" /><img />' )
        self.assertEqual( scrubHTML('<meta name="title" content="" /><meta>'),
                          '<meta name="title" content="" /><meta />' )

    def test_scrubHTML_with_adapter(self):
        from zope.component.testing import setUp
        from zope.component.testing import tearDown
        from zope.component import getSiteManager
        from zope.interface import implements

        from Products.CMFDefault.interfaces import IHTMLScrubber
        from Products.CMFDefault.utils import scrubHTML

        class _Scrubber:
            implements(IHTMLScrubber)
            def scrub(self, html):
                return html.upper()


        setUp()

        sm = getSiteManager()
        try:
            sm.registerUtility(_Scrubber(), IHTMLScrubber)
            self.assertEqual( scrubHTML('<a href="foo.html">bar</a>'),
                            '<A HREF="FOO.HTML">BAR</A>' )
            self.assertEqual( scrubHTML('<b>bar</b>'),
                            '<B>BAR</B>' )
            self.assertEqual( scrubHTML('<base href="" /><base>'),
                            '<BASE HREF="" /><BASE>' )
            self.assertEqual( scrubHTML('<blockquote>bar</blockquote>'),
                            '<BLOCKQUOTE>BAR</BLOCKQUOTE>' )
            self.assertEqual( scrubHTML('<body bgcolor="#ffffff">bar</body>'),
                            '<BODY BGCOLOR="#FFFFFF">BAR</BODY>' )
            self.assertEqual( scrubHTML('<br /><br>'),
                            '<BR /><BR>' )
            self.assertEqual( scrubHTML('<hr /><hr>'),
                            '<HR /><HR>' )
            self.assertEqual( scrubHTML('<img src="foo.png" /><img>'),
                            '<IMG SRC="FOO.PNG" /><IMG>' )
            self.assertEqual( scrubHTML(
                                '<meta name="title" content="" /><meta>'),
                            '<META NAME="TITLE" CONTENT="" /><META>' )

        finally:
            tearDown()

    def test_bodyfinder(self):
        from Products.CMFCore.tests.base.content import FAUX_HTML_LEADING_TEXT
        from Products.CMFCore.tests.base.content import SIMPLE_HTML
        from Products.CMFCore.tests.base.content import SIMPLE_STRUCTUREDTEXT
        from Products.CMFCore.tests.base.content import SIMPLE_XHTML
        from Products.CMFCore.tests.base.content import STX_WITH_HTML
        from Products.CMFDefault.utils import bodyfinder

        self.assertEqual( bodyfinder(FAUX_HTML_LEADING_TEXT),
                          '\n  <h1>Not a lot here</h1>\n ' )
        self.assertEqual( bodyfinder(SIMPLE_HTML),
                          '\n  <h1>Not a lot here</h1>\n ' )
        self.assertEqual( bodyfinder(SIMPLE_STRUCTUREDTEXT),
                          SIMPLE_STRUCTUREDTEXT )
        self.assertEqual( bodyfinder(SIMPLE_XHTML),
                          '\n  <h1>Not a lot here</h1>\n ' )
        self.assertEqual( bodyfinder(STX_WITH_HTML),
                          '<p>Hello world, I am Bruce.</p>' )

    def test_html_headcheck(self):
        from Products.CMFCore.tests.base.content import FAUX_HTML_LEADING_TEXT
        from Products.CMFCore.tests.base.content import SIMPLE_HTML
        from Products.CMFCore.tests.base.content import SIMPLE_STRUCTUREDTEXT
        from Products.CMFCore.tests.base.content import SIMPLE_XHTML
        from Products.CMFCore.tests.base.content import STX_WITH_HTML
        from Products.CMFDefault.utils import html_headcheck

        self.assertEqual( html_headcheck(FAUX_HTML_LEADING_TEXT), 0 )
        self.assertEqual( html_headcheck(SIMPLE_HTML), 1 )
        self.assertEqual( html_headcheck(SIMPLE_STRUCTUREDTEXT), 0 )
        self.assertEqual( html_headcheck(SIMPLE_XHTML), 1 )
        self.assertEqual( html_headcheck(STX_WITH_HTML), 0 )

    def test_tuplize(self):
        from Products.CMFDefault.utils import comma_split
        from Products.CMFDefault.utils import tuplize
        wanted = ('one','two','three')

        self.assertEqual( tuplize('string', 'one two three'), wanted )
        self.assertEqual( tuplize('unicode', u'one two three'), wanted )
        self.assertEqual( tuplize('string', 'one,two,three', comma_split),
                          wanted )
        self.assertEqual( tuplize('list', ['one',' two','three ']), wanted )
        self.assertEqual( tuplize('tuple', ('one','two','three')), wanted )

    def test_seq_strip(self):
        from Products.CMFDefault.utils import seq_strip

        self.assertEqual( seq_strip(['one ', ' two', ' three ']),
                          ['one','two','three'] )
        self.assertEqual( seq_strip(('one ', ' two', ' three ')),
                          ('one','two','three') )

    def test_html_marshal(self):
        from Products.CMFDefault.utils import html_marshal

        self.assertEqual( html_marshal(foo=1), ( ('foo:int', 1), ) )
        self.assertEqual( html_marshal(foo=1, bar='baz >&baz'),
                          ( ('foo:int', 1), ('bar', 'baz >&baz') ) )

    def test_toUnicode(self):
        from Products.CMFDefault.utils import toUnicode

        self.assertEqual( toUnicode('foo'), u'foo' )
        self.assertEqual( toUnicode( ('foo', 'bar'), 'ascii' ),
                          (u'foo', u'bar') )
        self.assertEqual( toUnicode( {'foo': 'bar'}, 'iso-8859-1' ),
                          {'foo': u'bar'} )

    def test_checkEmailAddress(self):
        from Products.CMFDefault.exceptions import EmailAddressInvalid
        from Products.CMFDefault.utils import checkEmailAddress

        self.assertEqual(checkEmailAddress('foo@example.com'), None)
        self.assertEqual(checkEmailAddress('foo@1bar.example.com'), None)
        self.assertEqual(checkEmailAddress('foo@123456.com'), None)
        self.assertEqual(checkEmailAddress('$.-@example.com'), None)
        self.assertEqual(checkEmailAddress(u'foo@example.com'), None)
        # CMF Collector issue #322
        self.assertEqual(checkEmailAddress('user+site@example.com'), None)
        # CMF Collector issue #326
        self.assertEqual(checkEmailAddress('username_@example.com'), None)
        # CMF Collector issue #401
        self.assertEqual(checkEmailAddress("user'site@example.com"), None)
        # CMF Collector issue #495
        self.assertEqual(checkEmailAddress("user@a.example.com"), None)
        self.assertRaises(EmailAddressInvalid, checkEmailAddress,
                          'this is not an e-mail address')
        self.assertRaises(EmailAddressInvalid, checkEmailAddress,
                          'foo@example.com, bar@example.com')
        self.assertRaises(EmailAddressInvalid, checkEmailAddress,
                          'foo.@example.com')
        self.assertRaises(EmailAddressInvalid, checkEmailAddress,
                          '.foo@example.com')
        self.assertRaises(EmailAddressInvalid, checkEmailAddress,
                          'foo@-bar.example.com')
        # RFC 2821 local-part: max 64 characters
        self.assertRaises(EmailAddressInvalid, checkEmailAddress,
                          'f'*63+'oo@example.com')
        # RFC 2821 domain: max 255 characters
        self.assertRaises(EmailAddressInvalid, checkEmailAddress,
                          'foo@'+'b'*242+'ar.example.com')

    def test_formatRFC822Headers_simple(self):
        from Products.CMFDefault.utils import formatRFC822Headers

        HEADERS = [ ('Foo', 'foo')
                  , ('Bar', 'bar')
                  ]

        formatted = formatRFC822Headers( HEADERS )

        self.assertEqual( formatted, 'Foo: foo\r\nBar: bar' )

    def test_formatRFC822Headers_empty(self):
        from Products.CMFDefault.utils import formatRFC822Headers

        HEADERS = [ ('Foo', 'foo')
                  , ('Bar', '')
                  ]

        formatted = formatRFC822Headers( HEADERS )

        self.assertEqual( formatted, 'Foo: foo\r\nBar: ' )

    def test_formatRFC822Headers_multiline(self):
        from Products.CMFDefault.utils import formatRFC822Headers

        HEADERS = [ ('Foo', 'foo')
                  , ('Bar', 'bar\nwith multiline')
                  ]

        formatted = formatRFC822Headers( HEADERS )

        self.assertEqual( formatted
                        , 'Foo: foo\r\nBar: bar\r\n  with multiline' )

    def test_formatRFC822Headers_multiline_trailing_blank_line(self):
        from Products.CMFDefault.utils import formatRFC822Headers

        HEADERS = [ ('Foo', 'foo')
                  , ('Bar', 'bar\nwith multiline\n')
                  ]

        formatted = formatRFC822Headers( HEADERS )

        self.assertEqual( formatted
                        , 'Foo: foo\r\nBar: bar\r\n  with multiline' )

    def test_formatRFC822Headers_multiline_intermediate_blank_line(self):
        from Products.CMFDefault.utils import formatRFC822Headers

        HEADERS = [ ('Foo', 'foo')
                  , ('Bar', 'bar\n\nwith multiline')
                  ]

        formatted = formatRFC822Headers( HEADERS )

        self.assertEqual( formatted
                        , 'Foo: foo\r\nBar: bar\r\n  \r\n  with multiline' )


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(DefaultUtilsTests),
        ))



##############################################################################
#
# Copyright (c) 2010 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Unit tests for the SyndicationInfo adapter.
"""

from datetime import datetime
import unittest
import Testing

from zope.annotation.interfaces import IAnnotations
from zope.component import getSiteManager
from zope.component import getAdapter
from zope.interface.verify import verifyClass

from Products.CMFCore.interfaces import ISyndicationTool
from Products.CMFCore.tests.base.testcase import TransactionalTest


class SyndicationInfoTests(TransactionalTest):

    def setUp(self):
        from zope.annotation.attribute import AttributeAnnotations
        from Products.CMFCore.interfaces import IFolderish
        from Products.CMFCore.PortalFolder import PortalFolder

        super(SyndicationInfoTests, self).setUp()
        self.app._setObject('portal', PortalFolder('portal'))
        self.portal = self.app.portal
        self.syndtool = DummySyndicationTool()
        sm = getSiteManager()
        sm.registerUtility(self.syndtool, ISyndicationTool)
        sm.registerAdapter(AttributeAnnotations, [IFolderish], IAnnotations)

    def _getTargetClass(self):
        from Products.CMFDefault.SyndicationInfo import SyndicationInfo

        return SyndicationInfo

    def _makeOne(self):
        from zope.interface import alsoProvides
        from Products.CMFCore.PortalFolder import PortalFolder
        folder = PortalFolder('folder')

        self.portal._setObject('folder', folder)
        alsoProvides(folder, IAnnotations)

        return self._getTargetClass()(folder)

    def test_interfaces(self):
        from Products.CMFCore.interfaces import ISyndicationInfo

        verifyClass(ISyndicationInfo, self._getTargetClass())

    def test_site_settings(self):
        adapter = self._makeOne()
        self.assertTrue(adapter.site_settings is self.syndtool)

    def test_set(self):
        adapter = self._makeOne()
        settings = {'max_items': 10, 'frequency': 7, 'period': 'daily',
                    'base': datetime.today()}
        annotations = getAdapter(adapter.context, IAnnotations)
        self.assertFalse(annotations.has_key(adapter.key))
        annotations[adapter.key] = settings
        for k, v in settings.items():
            self.assertEqual(getattr(adapter, k), v)

    def revert(self):
        adapter = self._makeOne()
        settings = {'max_items': 20, 'frequency': 1, 'period': 'monthly',
                    'base': datetime.today()}
        setattr(adapter.context, adapter.key, settings)
        self.assertEqual(getattr(adapter.context, adapter.key, settings))
        adapter.revert()
        self.assertNotEqual(getattr(adapter.context, adapter.key, settings))

    def test_not_enabled_by_default(self):
        adapter = self._makeOne()
        self.assertFalse(adapter.enabled)

    def test_enable(self):
        adapter = self._makeOne()
        self.syndtool.enabled = True
        adapter.enable()
        self.assertTrue(adapter.enabled)

    def test_get_default_values(self):
        adapter = self._makeOne()
        self.assertFalse(hasattr(adapter.context, adapter.key))
        self.assertEqual(adapter.period, self.syndtool.period)
        self.assertEqual(adapter.frequency, self.syndtool.frequency)
        self.assertEqual(adapter.base, self.syndtool.base)
        self.assertEqual(adapter.max_items, self.syndtool.max_items)

    def test_disable(self):
        adapter = self._makeOne()
        self.syndtool.enabled = True
        adapter.enable()
        self.assertTrue(adapter.enabled)
        adapter.disable()
        self.assertFalse(adapter.disable())


class DummySyndicationTool(object):

    enabled  = 0
    period = 'daily'
    frequency = 1
    base = datetime(2010, 10, 3, 12, 0, 0)
    max_items = 15


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(SyndicationInfoTests),
        ))



##parameters=include_root=1
##title=Return breadcrumbs
##
from Products.CMFCore.utils import getUtilityByInterfaceName
from Products.CMFDefault.utils import Message as _

ptool = getUtilityByInterfaceName('Products.CMFCore.interfaces.IPropertiesTool')
utool = getUtilityByInterfaceName('Products.CMFCore.interfaces.IURLTool')
portal_url = utool()
result = []

if include_root:
    result.append( { 'id'      : _(u'root')
                   , 'title'   : ptool.title()
                   , 'url'     : portal_url
                   }
                 )

relative = utool.getRelativeContentPath(context)
portal = utool.getPortalObject()

for i in range( len( relative ) ):
    now = relative[ :i+1 ]
    obj = portal.restrictedTraverse( now )
    if not now[ -1 ] == 'talkback':
        result.append( { 'id'      : now[ -1 ]
                       , 'title'   : obj.Title()
                       , 'url'     : portal_url + '/' + '/'.join(now)
                       }
                    )

return result



##############################################################################
#
# Copyright (c) 2010 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Change password form.
"""

from DateTime import DateTime
from zope.component import adapts
from zope.component import getUtility
from zope.formlib import form
from zope.interface import implements
from zope.interface import Interface
from zope.interface import Invalid
from zope.interface import invariant
from zope.schema import Password

from Products.CMFCore.interfaces import IMember
from Products.CMFCore.interfaces import IMembershipTool
from Products.CMFCore.interfaces import IPropertiesTool
from Products.CMFCore.interfaces import ISiteRoot
from Products.CMFDefault.browser.utils import decode
from Products.CMFDefault.browser.utils import memoize
from Products.CMFDefault.formlib.form import SettingsEditFormBase
from Products.CMFDefault.utils import Message as _


class IPasswordSchema(Interface):

    password = Password(
                    title=_(u"New password"),
                    min_length=5
                    )

    confirmation = Password(
                    title=_(u"Confirm new password"),
                    required=False
                    )

    @invariant
    def check_passwords_match(schema):
        """Password and confirmation must match"""
        if schema.password != schema.confirmation:
            raise Invalid(_(u"Your password and confirmation did not match. "
                            u"Please try again."))


class PasswordSchemaAdapter(object):

    adapts(IMember)
    implements(IPasswordSchema)

    def __init__(self, context):
        self.context = context

    def _getPassword(self):
        return u''

    def _setPassword(self, val):
        ptool = getUtility(IPropertiesTool)
        default_charset = ptool.getProperty('default_charset', None)
        self.context.setSecurityProfile(val.encode(default_charset))

    def _getLastLoginTime(self):
        return self.context.getProperty('last_login_time')

    def _setLastLoginTime(self, val):
        self.context.setProperties(last_login_time=val)

    password = property(_getPassword, _setPassword)
    confirmation = u''
    last_login_time = property(_getLastLoginTime, _setLastLoginTime)


class PasswordFormView(SettingsEditFormBase):

    """Edit view for IPasswordSchema.

    Apply same constraints as when joining. Users that have been sent a
    password must change it using this form the first time they log in.
    """

    successMessage = _(u"Your password has been changed.")
    form_fields = form.FormFields(IPasswordSchema)

    @property
    def label(self):
        if self.is_first_login:
            return _(u'Welcome!')
        else:
            return _(u'Change your Password')

    @property
    @decode
    def portal_title(self):
        return getUtility(ISiteRoot).title

    @property
    def description(self):
        if self.is_first_login:
            return _(u"This is the first time that you've logged in to "
                     u"${portal_title}. Before you start exploring you need "
                     u"to change your original password. This will ensure "
                     u"that the password we sent you via email cannot be "
                     u"used in a malicious manner.",
                     mapping={'portal_title': self.portal_title})
        else:
            return _(u'Please use the form below to change your password.')

    @memoize
    def getContent(self):
        mtool = getUtility(IMembershipTool)
        member = mtool.getAuthenticatedMember()
        return PasswordSchemaAdapter(member)

    @property
    def is_first_login(self):
        return self.getContent().last_login_time == DateTime('1999/01/01')

    def applyChanges(self, data):
        changes = super(PasswordFormView, self).applyChanges(data)
        if self.getContent().last_login_time == DateTime('1999/01/01'):
            self.getContent().last_login_time = DateTime('2000/01/01')
        mtool = getUtility(IMembershipTool)
        mtool.credentialsChanged(self.getContent().context.getPassword(),
                                 self.request)
        return changes

    def handle_change_success(self, action, data):
        self._handle_success(action, data)
        return self._setRedirect('portal_actions', 'user/mystuff')

    def handle_cancel_success(self, action, data):
        return self._setRedirect('portal_actions', 'user/mystuff')



##############################################################################
#
# Copyright (c) 2009 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Upgrade steps to CMFDefault 2.0.

$Id: to20.py 110420 2010-04-01 17:12:56Z tseaver $
"""
import logging

from Acquisition import aq_base
from Acquisition import aq_inner
from Acquisition import aq_parent

from Products.CMFCore.utils import getToolByName
from Products.CMFDefault.MetadataTool import _DCMI_ELEMENT_SPECS
from Products.CMFDefault.MetadataTool import MetadataSchema


def check_dcmi_metadata(tool):
    """1.6.x to 2.0.0 upgrade step checker
    """
    metadata_tool = getToolByName(tool, 'portal_metadata')
    return getattr(aq_base(metadata_tool), 'element_specs', None) is not None

def upgrade_dcmi_metadata(tool):
    """1.6.x to 2.0.0 upgrade step handler
    """
    logger = logging.getLogger('GenericSetup.upgrade')
    metadata_tool = getToolByName(tool, 'portal_metadata')
    old_specs = getattr(metadata_tool, 'element_specs', None)
    if old_specs is not None:
        del metadata_tool.element_specs
        dcmi = metadata_tool._DCMI = MetadataSchema('DCMI', _DCMI_ELEMENT_SPECS)
        for element_id, old_spec in old_specs.items():
            new_spec = dcmi.getElementSpec(element_id)
            for typ, policy in old_spec.listPolicies():
                if typ is not None:
                    new_spec.addPolicy(typ)
                tp = new_spec.getPolicy(typ)
                tp.edit( is_required=policy.isRequired()
                       , supply_default=policy.supplyDefault()
                       , default_value=policy.defaultValue()
                       , enforce_vocabulary=policy.enforceVocabulary()
                       , allowed_vocabulary=policy.allowedVocabulary()
                       )
    logger.info('Dublin Core metadata definition updated.')

def check_newstyle_actions(tool):
    """1.6.x to 2.0.0 upgrade step checker
    """
    portal = aq_parent(aq_inner(tool))
    if not portal.portal_actions.objectIds(['CMF Action Category']):
        return True

    return False

def upgrade_to_newstyle_actions(tool):
    """1.6.x to 2.0.0 upgrade step handler
    """
    logger = logging.getLogger('GenericSetup.upgrade')
    portal = aq_parent(aq_inner(tool))
    if not portal.portal_actions.objectIds(['CMF Action Category']):
        tool.runImportStepFromProfile( 'profile-Products.CMFDefault:default'
                                     , 'actions'
                                     )
        logger.info('Instantiated new-style actions in portal_actions')



##############################################################################
#
# Copyright (c) 2008 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Tests for membership views """

import unittest
from Testing import ZopeTestCase

from DateTime.DateTime import DateTime
from zope.component import getSiteManager
from zope.publisher.browser import TestRequest
from zope.publisher.interfaces.browser import IBrowserPublisher
from zope.testing.cleanup import cleanUp

from Products.CMFCore.interfaces import IActionsTool
from Products.CMFCore.interfaces import IMembershipTool
from Products.CMFCore.interfaces import IURLTool
from Products.CMFCore.tests.base.dummy import DummySite
from Products.CMFCore.tests.base.dummy import DummyTool
from Products.CMFCore.tests.base.dummy import DummyUser
from Products.CMFDefault.browser.membership.members import Manage
from Products.CMFDefault.browser.membership.members import MemberProxy
from Products.CMFDefault.testing import FunctionalLayer


class DummyUser(DummyUser):

    def getProperty(self, attr):
        if attr == 'login_time':
            return DateTime('2000/01/01 00:00:00')
        elif attr == 'fullname':
            return 'FULL NAME'
        return None


class DummyMemberTool(object):

    def __init__(self):
        self.members = {}

    def listMembers(self):
        return self.members.values()

    def addMember(self, member):
        self.members[member.getId()] = member

    def deleteMembers(self, member_ids):
        for i in member_ids:
            del self.members[i]

    def isAnonymousUser(self):
        return True

    def getHomeUrl(self, id=None, verifyPermission=0):
        return 'HOME_URL/%s' % id


class MembershipViewTests(unittest.TestCase):

    def setUp(self):
        """Setup a site"""
        self.site = DummySite('site')
        self.mtool = DummyMemberTool()
        sm = getSiteManager()
        sm.registerUtility(DummyTool(), IActionsTool)
        sm.registerUtility(self.mtool, IMembershipTool)
        sm.registerUtility(DummyTool().__of__(self.site), IURLTool)

    def tearDown(self):
        cleanUp()

    def _make_one(self, name="DummyUser"):
        user = DummyUser(name)
        self.mtool.addMember(user)
        return user

    def _make_batch(self):
        """Add enough objects to force pagination"""
        batch_size = Manage._BATCH_SIZE
        for i in range(batch_size + 2):
            user_id = "Dummy%s" % i
            self._make_one(user_id)

    def test_getNavigationURL(self):
        url = 'http://example.com/members.html'
        self._make_batch()
        view = Manage(self.site, TestRequest(ACTUAL_URL=url))
        self.assertTrue(view._getNavigationURL(25) == url + "?form.b_start=25")

    def test_view(self):
        view = Manage(self.site, TestRequest())
        self.assertTrue(IBrowserPublisher.providedBy(view))

    def test_list_batch_items(self):
        self._make_one("Bob")
        view = Manage(self.site, TestRequest())
        view.member_fields()
        members = view.listBatchItems
        self.assertTrue(isinstance(members[0], MemberProxy))
        self.assertEqual(members[0].name, 'FULL NAME (Bob)')
        self.assertEqual(members[0].home, "HOME_URL/Bob")

    def test_get_ids(self):
        view = Manage(self.site, TestRequest())
        self.assertEquals(
                        view._get_ids({'foo':'bar'}),
                        [])
        self.assertEquals(
                        sorted(
                            view._get_ids({'DummyUser1.select':True,
                                       'DummyUser2.select':False,
                                       'DummyUser3.select':True})
                            ),
                        ['DummyUser1', 'DummyUser3']
                        )
        self.assertEquals(
                        view._get_ids({'stupid.name.select.select':True}),
                        ['stupid.name.select']
        )

    def test_handle_select_for_deletion(self):
        view = Manage(self.site, TestRequest())
        self.assertTrue(view.guillotine == None)
        # Catch exception raised when template tries to render
        self.assertRaises(AttributeError,
                view.handle_select_for_deletion, None, {"Alice.select":True})
        self.assertTrue(view.guillotine == "Alice")

    def test_handle_delete(self):
        self._make_one("Bob")
        view = Manage(self.site, TestRequest())
        self.assertFalse(self.mtool.listMembers() == [])
        # Catch exception raised when trying to redirect
        self.assertRaises(TypeError,
                         view.handle_delete, None, {"Bob.select":True}
                         )
        self.assertTrue(self.mtool.listMembers() == [])


ftest_suite = ZopeTestCase.FunctionalDocFileSuite('members.txt')
ftest_suite.layer = FunctionalLayer

def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(MembershipViewTests))
    suite.addTest(unittest.TestSuite((ftest_suite,)))
    return suite



##parameters=b_start=0
##
from Products.PythonScripts.standard import thousands_commas
from ZTUtils import Batch
from Products.CMFCore.utils import getUtilityByInterfaceName
from Products.CMFDefault.utils import decode
from Products.CMFDefault.utils import Message as _

ctool = getUtilityByInterfaceName('Products.CMFCore.interfaces.ICatalogTool')
utool = getUtilityByInterfaceName('Products.CMFCore.interfaces.IURLTool')
portal_url = utool()
epoch = DateTime('1970/01/01 00:00:01 GMT')


options = {}

target = '%s/search' % portal_url
kw = context.REQUEST.form.copy()
for k, v in kw.items():
    if k in ('review_state', 'Title', 'Subject', 'Description', 'portal_type',
             'listCreators'):
        if same_type(v, []):
            v = filter(None, v)
        if not v:
            del kw[k]
    elif k in ('created',):
        if v['query'] == epoch and v['range'] == 'min':
            del kw[k]
        else:
            # work around problems with DateTime in records
            kw[k] = v.copy()
    elif k in ('go', 'go.x', 'go.y', 'b_start'):
            del kw[k]
items = ctool.searchResults(kw)
batch_obj = Batch(items, 25, b_start, orphan=1)

items = [ {'description': item.Description,
           'icon': item.getIconURL,
           'title': item.Title,
           'type': item.Type,
           'date': item.Date,
           'url': item.getURL()+'/view'}
          for item in batch_obj ]

length = batch_obj.sequence_length
summary = { 'length': length and thousands_commas(length) or '',
            'type': (length == 1) and _(u'item') or _(u'items'),
            'match': kw.get('SearchableText') }
navigation = context.getBatchNavigation(batch_obj, target, **kw)
options['batch'] = { 'summary': summary,
                     'listItemInfos': tuple(items),
                     'navigation': navigation }

return context.search_results_template(**decode(options, script))



##############################################################################
#
# Copyright (c) 2001 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Link instances represent explicit links-as-content.
"""

import urlparse

from AccessControl.SecurityInfo import ClassSecurityInfo
from App.class_init import InitializeClass
from App.special_dtml import DTMLFile
import transaction
from zope.component.factory import Factory
from zope.interface import implements

from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore.utils import contributorsplitter
from Products.CMFCore.utils import keywordsplitter
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CMFDefault.exceptions import ResourceLockedError
from Products.CMFDefault.interfaces import ILink
from Products.CMFDefault.interfaces import IMutableLink
from Products.CMFDefault.permissions import ModifyPortalContent
from Products.CMFDefault.permissions import View
from Products.CMFDefault.utils import _dtmldir
from Products.CMFDefault.utils import formatRFC822Headers
from Products.CMFDefault.utils import parseHeadersBody
from Products.GenericSetup.interfaces import IDAVAware


def addLink( self
           , id
           , title=''
           , remote_url=''
           , description=''
           ):
    """Add a Link instance to 'self'.
    """
    o=Link( id, title, remote_url, description )
    self._setObject(id, o, suppress_events=True)


class Link(PortalContent, DefaultDublinCoreImpl):

    """A Link.
    """

    implements(IMutableLink, ILink, IDAVAware)

    URL_FORMAT = format = 'text/url'
    effective_date = expiration_date = None

    security = ClassSecurityInfo()

    def __init__( self
                , id
                , title=''
                , remote_url=''
                , description=''
                ):
        DefaultDublinCoreImpl.__init__(self)
        self.id=id
        self.title=title
        self.description=description
        self._edit(remote_url)
        self.format=self.URL_FORMAT

    security.declareProtected(ModifyPortalContent, 'manage_edit')
    manage_edit = DTMLFile( 'zmi_editLink', _dtmldir )

    security.declareProtected(ModifyPortalContent, 'manage_editLink')
    def manage_editLink( self, remote_url, REQUEST=None ):
        """
            Update the Link via the ZMI.
        """
        self._edit( remote_url )
        if REQUEST is not None:
            REQUEST['RESPONSE'].redirect( self.absolute_url()
                                        + '/manage_edit'
                                        + '?manage_tabs_message=Link+updated'
                                        )

    security.declarePrivate( '_edit' )
    def _edit( self, remote_url ):
        """
            Edit the Link
        """
        tokens = urlparse.urlparse( remote_url, 'http' )
        if tokens[0] == 'http':
            if tokens[1]:
                # We have a nethost. All is well.
                url = urlparse.urlunparse(tokens)
            elif tokens[2:] == ('', '', '', ''):
                # Empty URL
                url = ''
            else:
                # Relative URL, keep it that way, without http:
                tokens = ('', '') + tokens[2:]
                url = urlparse.urlunparse(tokens)
        else:
            # Other scheme, keep original
            url = urlparse.urlunparse(tokens)
        self.remote_url = url

    security.declareProtected(ModifyPortalContent, 'edit')
    def edit(self, remote_url ):
        """ Update and reindex. """
        self._edit( remote_url )
        self.reindexObject()

    security.declareProtected(View, 'SearchableText')
    def SearchableText(self):
        """
            text for indexing
        """
        return "%s %s" % (self.title, self.description)

    security.declareProtected(View, 'getRemoteUrl')
    def getRemoteUrl(self):
        """
            returns the remote URL of the Link
        """
        return self.remote_url

    security.declarePrivate('_writeFromPUT')
    def _writeFromPUT(self, body):
        headers, body = parseHeadersBody(body)
        lines = body.split('\n')
        self.edit(lines[0])
        headers['Format'] = self.URL_FORMAT
        new_subject = keywordsplitter(headers)
        headers['Subject'] = new_subject or self.Subject()
        new_contrib = contributorsplitter(headers)
        headers['Contributors'] = new_contrib or self.Contributors()
        headers = dict((k.lower(), v) for k, v in headers.iteritems())
        self._editMetadata(**headers)

    ## FTP handlers
    security.declareProtected(ModifyPortalContent, 'PUT')
    def PUT(self, REQUEST, RESPONSE):
        """
            Handle HTTP / WebDAV / FTP PUT requests.
        """
        self.dav__init(REQUEST, RESPONSE)
        self.dav__simpleifhandler(REQUEST, RESPONSE, refresh=1)
        body = REQUEST.get('BODY', '')
        try:
            self._writeFromPUT( body )
            RESPONSE.setStatus(204)
            return RESPONSE
        except ResourceLockedError, msg:
            transaction.abort()
            RESPONSE.setStatus(423)
            return RESPONSE

    security.declareProtected(View, 'manage_FTPget')
    def manage_FTPget(self):
        """
            Get the link as text for WebDAV src / FTP download.
        """
        hdrlist = self.getMetadataHeaders()
        hdrtext = formatRFC822Headers( hdrlist )
        bodytext = '%s\n\n%s' % ( hdrtext, self.getRemoteUrl() )

        return bodytext

    security.declareProtected(View, 'get_size')
    def get_size( self ):
        """ Used for FTP and apparently the ZMI now too.
        """
        return len(self.manage_FTPget())

InitializeClass(Link)

LinkFactory = Factory(Link)



##############################################################################
#
# Copyright (c) 2010 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Workflow forms.
"""

from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.component import getUtility
from zope.formlib import form
from zope.interface import Interface
from zope.schema import Text

from Products.CMFCore.interfaces import IWorkflowTool
from Products.CMFDefault.browser.utils import memoize
from Products.CMFDefault.exceptions import WorkflowException
from Products.CMFDefault.formlib.form import EditFormBase
from Products.CMFDefault.utils import Message as _

STATUS = {'submit': _(u"Item submitted for review."),
          'publish': _(u"Item was published."),
          'retract': _(u"Item was retracted."),
          'reject': _(u"Item was rejected."),
          'hide': _("Item was hidden."),
          'show': _("Item is now visible.")
         }


class IWorkflowSchema(Interface):

    comment = Text(
                title=u"comments",
                required=False
                )


class Submit(EditFormBase):

    """Submit an item for review"""

    template = ViewPageTemplateFile("submit.pt")
    form_fields = form.FormFields(IWorkflowSchema)
    actions = form.Actions(
        form.Action(
            name="submit",
            label=_(u"Submit item"),
            success='handle_workflow',
            failure='handle_failure'
            )
        )

    @property
    @memoize
    def workflow(self):
        return getUtility(IWorkflowTool)

    def handle_workflow(self, action, data):
        try:
            self.workflow.doActionFor(self.context, action.name,
                                      comment=data['comment'])
            self.status = STATUS.get(action.name, _(u"Status changed."))
            self._setRedirect('portal_types', 'object/view')
        except WorkflowException, errmsg:
            self.status = errmsg
            self._setRedirect('portal_actions', 'object/edit')


class Publish(Submit):

    """Publish an item"""

    template = ViewPageTemplateFile("publish.pt")
    form_fields = form.FormFields(IWorkflowSchema)
    actions = form.Actions(
        form.Action(
            name="publish",
            label=_(u"Publish this item"),
            success='handle_workflow',
            failure='handle_failure'
            )
        )


class Retract(Submit):

    """Remove a published object"""

    template = ViewPageTemplateFile("retract.pt")
    form_fields = form.FormFields(IWorkflowSchema)
    actions = form.Actions(
        form.Action(
            name="retract",
            label=_(u"Retract this item"),
            success='handle_workflow',
            failure='handle_failure'
            )
        )


class Reject(Submit):

    """Reject an item submitted for publication"""

    template = ViewPageTemplateFile("reject.pt")
    form_fields = form.FormFields(IWorkflowSchema)
    actions = form.Actions(
        form.Action(
            name="reject",
            label=_(u"Reject this item"),
            success='handle_workflow',
            failure='handle_failure'
            )
        )

class Hide(Submit):

    """Hide a published item"""

    template = ViewPageTemplateFile("hide.pt")
    form_fields = form.FormFields(IWorkflowSchema)
    actions = form.Actions(
        form.Action(
            name="hide",
            label=_(u"Hide this item"),
            success='handle_workflow',
            failure='handle_failure'
            )
        )

class Show(Submit):

    """Reveal a hidden item"""

    template = ViewPageTemplateFile("show.pt")
    form_fields = form.FormFields(IWorkflowSchema)
    actions = form.Actions(
        form.Action(
            name="show",
            label=_(u"Show this item"),
            success='handle_workflow',
            failure='handle_failure'
            )
        )



##############################################################################
#
# Copyright (c) 2009 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Shared utility functions for browser view tests
"""

from Zope2.App.schema import Zope2VocabularyRegistry

def setupVocabulary(testcase):
    from zope.schema.vocabulary import setVocabularyRegistry
    setVocabularyRegistry(Zope2VocabularyRegistry())

def clearVocabulary(testcase):
    from zope.schema.vocabulary import _clear
    _clear()



##############################################################################
#
# Copyright (c) 2010 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Portal Configuration Form.
"""

from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.component import getUtility
from zope.formlib import form
from zope.schema import getFieldNames

from Products.CMFCore.interfaces import IPropertiesTool
from Products.CMFDefault.formlib.form import EditFormBase
from Products.CMFDefault.formlib.widgets import ChoiceRadioWidget
from Products.CMFDefault.utils import Message as _

from .interfaces import IPortalConfig


class PortalConfig(EditFormBase):

    form_fields = form.FormFields(IPortalConfig)
    form_fields['validate_email'].custom_widget = ChoiceRadioWidget

    actions = form.Actions(
        form.Action(
            name='change',
            label=_(u'Change'),
            success='handle_success',
            failure='handle_failure'),
    )
    template = ViewPageTemplateFile("config.pt")

    def setUpWidgets(self, ignore_request=False):
        data = {}
        ptool = getUtility(IPropertiesTool)
        charset = ptool.getProperty('default_charset', None)
        for name in getFieldNames(IPortalConfig):
            value = ptool.getProperty(name)
            try:
                value = value.decode(charset)
            except (AttributeError, UnicodeEncodeError):
                pass
            data[name] = value
        data['smtp_server'] = ptool.smtp_server()
        self.widgets = form.setUpDataWidgets(
                    self.form_fields, self.prefix,
                    self.context, self.request, data=data,
                    ignore_request=ignore_request)

    def handle_success(self, action, data):
        ptool = getUtility(IPropertiesTool)
        ptool.editProperties(data)
        self.status = _(u"Portal settings changed.")
        self._setRedirect('portal_actions', 'global/configPortal')



##############################################################################
#
# Copyright (c) 2005 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Unit tests for SyndicationTool module.
"""
from datetime import datetime
from datetime import timedelta
import unittest
import Testing

from DateTime.DateTime import DateTime
from zope.component import getSiteManager
from zope.component import queryAdapter
from zope.interface import alsoProvides
from zope.interface.verify import verifyClass
from zope.testing.cleanup import cleanUp

from Products.CMFCore.interfaces import ITypesTool
from Products.CMFCore.tests.base.testcase import SecurityTest


class Dummy:

    period = None
    frequency = None
    base = None
    max_items = None

    def getId(self):
        return 'dummy'


class DummyInfo(object):

    def __init__(self, context):
        self.context = context

    def __call__(self):
        pass

    def set_f(self, frequency):
        self.context.frequency = frequency

    def get_f(self):
        return self.context.frequency

    frequency = property(get_f, set_f)

    def get_p(self):
        return self.context.period

    def set_p(self, period):
        self.context.period = period

    period = property(get_p, set_p)

    def get_b(self):
        return self.context.base

    def set_b(self, base):
        self.context.base = base

    base = property(get_b, set_b)

    def get_m(self):
        return self.context.max_items

    def set_m(self, max_items):
        self.context.max_items = max_items

    max_items = property(get_m, set_m)

    @property
    def enabled(self):
        if hasattr(self.context, 'enabled'):
            return self.context.enabled
        return False

    def enable(self):
        self.context.enabled = True

    def disable(self):
        self.context.enabled = False


class SyndicationToolTests(SecurityTest):

    def _getTargetClass(self):
        from Products.CMFDefault.SyndicationTool import SyndicationTool
        return SyndicationTool

    def _makeOne(self, *args, **kw):
        return self._getTargetClass()(*args, **kw)

    def _makeContext(self):
        from Products.CMFCore.interfaces import IFolderish, ISyndicationInfo
        self.folder = folder = Dummy()
        alsoProvides(folder, IFolderish)
        sm = getSiteManager()
        sm.registerAdapter(DummyInfo, [IFolderish], ISyndicationInfo)
        return folder

    def _makeInfo(self, context):
        info = DummyInfo(context)
        return info

    def tearDown(self):
        cleanUp()
        SecurityTest.tearDown(self)

    def test_interfaces(self):
        from Products.CMFCore.interfaces import ISyndicationTool

        verifyClass(ISyndicationTool, self._getTargetClass())

    def test_empty(self):
        ONE_MINUTE = timedelta(0, 61, 0)

        tool = self._makeOne()

        self.assertEqual(tool.period, 'daily')
        self.assertEqual(tool.frequency, 1)
        self.failUnless(datetime.now() - tool.base < ONE_MINUTE)
        self.failIf(tool.enabled)
        self.assertEqual(tool.max_items, 15)

    def test_object_not_syndicatable(self):
        tool = self._makeOne()
        self.assertFalse(tool.isSyndicationAllowed(Dummy))

    def test_object_is_syndicatable(self):
        from Products.CMFCore.interfaces import ISyndicationInfo
        self._makeOne()
        context = self._makeContext()
        adapter = queryAdapter(context, ISyndicationInfo)
        self.assertTrue(adapter is not None)

    def test_object_syndication_is_disabled(self):
        tool = self._makeOne()
        context = self._makeContext()
        self.assertFalse(tool.isSyndicationAllowed(context))

    def test_enable_object_syndication(self):
        tool = self._makeOne()
        tool.enabled = True
        context = self._makeContext()
        tool.enableSyndication(context)
        self.assertTrue(tool.isSyndicationAllowed(context))

    def test_editSyInformationProperties_normal(self):
        PERIOD = 'hourly'
        FREQUENCY = 4
        NOW = datetime.now()
        MAX_ITEMS = 42

        tool = self._makeOne()
        tool.enabled = True
        context = self._makeContext()
        tool.enableSyndication(context)

        tool.editSyInformationProperties(context,
                                         updatePeriod=PERIOD,
                                         updateFrequency=FREQUENCY,
                                         updateBase=NOW,
                                         max_items=MAX_ITEMS,
                                        )
        info = tool.getSyndicationInfo(context)
        self.assertEqual(info.frequency, FREQUENCY)
        self.assertEqual(info.period, PERIOD)
        self.assertEqual(info.base, NOW)
        self.assertEqual(info.max_items, MAX_ITEMS)

    def test_getSyndicatableContent(self):
        # http://www.zope.org/Collectors/CMF/369
        # Make sure we use a suitable base class call when determining
        # syndicatable content
        from Products.CMFCore.PortalFolder import PortalFolder
        from Products.CMFCore.CMFBTreeFolder import CMFBTreeFolder
        from Products.CMFCore.TypesTool import TypesTool

        PERIOD = 'hourly'
        FREQUENCY = 4
        NOW = datetime.now()
        MAX_ITEMS = 42

        getSiteManager().registerUtility(TypesTool(), ITypesTool)
        self.app._setObject('pf', PortalFolder('pf'))
        self.app._setObject('bf', CMFBTreeFolder('bf'))
        tool = self._makeOne()
        tool.period = PERIOD
        tool.frequency = FREQUENCY
        tool.base = NOW
        tool.enabled = True
        tool.max_items = MAX_ITEMS

        self.assertEqual(len(tool.getSyndicatableContent(self.app.pf)), 0)
        self.assertEqual(len(tool.getSyndicatableContent(self.app.bf)), 0)

    def test_getUpdateBase(self):
        NOW = datetime.now()

        tool = self._makeOne()
        tool.base = NOW

        self.assertEqual(NOW.isoformat(), tool.getUpdateBase())


    def test_getUpdateBaseWithContext(self):
        NOW = datetime.now()

        tool = self._makeOne()
        tool.enabled = True

        context = self._makeContext()
        info = self._makeInfo(context)
        tool.enableSyndication(context)
        info.base = NOW

        self.assertEqual(NOW.isoformat(), tool.getUpdateBase(context))


    def test_getHTML4UpdateBase(self):
        NOW = datetime.now()

        tool = self._makeOne()
        tool.base = NOW

        as_HTML4 = tool.getHTML4UpdateBase()
        self.assertEqual(NOW.strftime("%Y-%m-%dT%H:%M:%SZ"), as_HTML4)

    def test_getHTML4UpdateBaseWithContext(self):
        NOW = datetime.now()

        tool = self._makeOne()
        tool.enabled = True

        context = self._makeContext()
        info = self._makeInfo(context)
        tool.enableSyndication(context)
        info.base = NOW

        as_HTML4 = tool.getHTML4UpdateBase(context)

        self.assertEqual(NOW.strftime("%Y-%m-%dT%H:%M:%SZ"), as_HTML4)

    def test_getMaxItems(self):
        max_items = 5

        tool = self._makeOne()
        tool.max_items = max_items

        self.assertEqual(max_items, tool.getMaxItems())

    def test_getMaxItemsWithContext(self):
        max_items = 10

        tool = self._makeOne()
        tool.enabled = True

        context = self._makeContext()
        info = self._makeInfo(context)
        tool.enableSyndication(context)
        info.max_items = max_items

        self.assertEqual(max_items, tool.getMaxItems(context))

    def test_getUpdatePeriod(self):
        period = 3

        tool = self._makeOne()
        tool.period = period

        self.assertEqual(period, tool.getUpdatePeriod())

    def test_getUpdatePeriodWithContext(self):
        period = 2

        tool = self._makeOne()
        tool.enabled = True

        context = self._makeContext()
        info = self._makeInfo(context)
        tool.enableSyndication(context)
        info.period = period

        self.assertEqual(period, tool.getUpdatePeriod(context))

    def test_getUpdateFrequency(self):
        frequency = 'monthly'

        tool = self._makeOne()
        tool.frequency = frequency

        self.assertEqual(frequency, tool.getUpdateFrequency())

    def test_getUpdateFrequencyWithContext(self):
        frequency = 'weekly'

        tool = self._makeOne()
        tool.enabled = True

        context = self._makeContext()
        info = self._makeInfo(context)
        tool.enableSyndication(context)
        info.frequency = frequency

        self.assertEqual(frequency, tool.getUpdateFrequency(context))


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(SyndicationToolTests),
        ))



##############################################################################
#
# Copyright (c) 2002 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Unit tests for MembershipTool module.
"""

import unittest
import Testing

from AccessControl.SecurityManagement import newSecurityManager
from five.localsitemanager import make_objectmanager_site
from zope.component import getSiteManager
from zope.component.hooks import setSite
from zope.globalrequest import clearRequest
from zope.globalrequest import setRequest
from zope.interface.verify import verifyClass
from zope.testing.cleanup import cleanUp

from Products.CMFCore.interfaces import IWorkflowTool
from Products.CMFCore.PortalFolder import PortalFolder
from Products.CMFCore.tests.base.dummy import DummyFolder
from Products.CMFCore.tests.base.dummy import DummySite
from Products.CMFCore.tests.base.dummy import DummyTool
from Products.CMFCore.tests.base.dummy import DummyUserFolder
from Products.CMFCore.tests.base.testcase import SecurityTest
from Products.CMFCore.tests.base.testcase import TransactionalTest


class MembershipToolTests(TransactionalTest):

    def _makeOne(self, *args, **kw):
        from Products.CMFDefault.MembershipTool import MembershipTool

        return MembershipTool(*args, **kw)

    def setUp(self):
        from Products.CMFCore.interfaces import IMembershipTool

        TransactionalTest.setUp(self)
        self.site = DummySite('site').__of__(self.app)
        make_objectmanager_site(self.site)
        setSite(self.site)
        self.site._setObject('portal_membership', self._makeOne())
        sm = getSiteManager()
        sm.registerUtility(self.site.portal_membership, IMembershipTool)

    def tearDown(self):
        cleanUp()
        TransactionalTest.tearDown(self)

    def test_interfaces(self):
        from Products.CMFDefault.interfaces import IMembershipTool
        from Products.CMFDefault.MembershipTool import MembershipTool

        verifyClass(IMembershipTool, MembershipTool)

    def test_MembersFolder_methods(self):
        from Products.CMFCore.interfaces import IMembershipTool

        mtool = getSiteManager().getUtility(IMembershipTool)
        self.assertEqual(mtool.getMembersFolder(), None)
        self.site._setObject('Members', DummyFolder())
        self.assertEqual(mtool.getMembersFolder(), self.site.Members)
        mtool.setMembersFolderById(id='foo')
        self.assertEqual(mtool.getMembersFolder(), None)
        self.site._setObject('foo', DummyFolder())
        self.assertEqual(mtool.getMembersFolder(), self.site.foo)
        mtool.setMembersFolderById(id='foo/members')
        self.assertEqual(mtool.getMembersFolder(), None)
        self.site.foo._setObject('members', DummyFolder())
        self.assertEqual(mtool.getMembersFolder(), self.site.foo.members)
        mtool.setMembersFolderById()
        # Note: self.site is returned due to DummyObject.restrictedTraverse
        self.assertEqual(mtool.getMembersFolder(), self.site)

    def test_HomeFolder_methods(self):
        from Products.CMFCore.interfaces import IMembershipTool

        mtool = getSiteManager().getUtility(IMembershipTool)
        setRequest(self.REQUEST)
        self.assertEqual(mtool.getHomeFolder(id='member_foo'), None)
        self.assertEqual(mtool.getHomeUrl(id='member_foo'), None)
        self.site._setObject('Members', PortalFolder('Members'))
        self.assertEqual(mtool.getHomeFolder(id='member_foo'), None)
        self.assertEqual(mtool.getHomeUrl(id='member_foo'), None)
        self.site.Members._setObject('member_foo', PortalFolder('member_foo'))
        self.assertEqual(mtool.getHomeFolder(id='member_foo'),
                         self.site.Members.member_foo)
        self.assertEqual(mtool.getHomeUrl(id='member_foo'),
                         'http://nohost/bar/site/Members/member_foo')
        clearRequest()


class MembershipToolSecurityTests(SecurityTest):

    def _makeOne(self, *args, **kw):
        from Products.CMFDefault.MembershipTool import MembershipTool

        return MembershipTool(*args, **kw)

    def setUp(self):
        SecurityTest.setUp(self)
        self.site = DummySite('site').__of__(self.app)
        self.site._setObject( 'portal_membership', self._makeOne() )

    def tearDown(self):
        cleanUp()
        SecurityTest.tearDown(self)

    def test_createMemberArea(self):
        mtool = self.site.portal_membership
        members = self.site._setObject( 'Members', PortalFolder('Members') )
        acl_users = self.site._setObject( 'acl_users', DummyUserFolder() )
        wtool = DummyTool()
        getSiteManager().registerUtility(wtool, IWorkflowTool)

        # permission
        mtool.createMemberArea('user_foo')
        self.failIf( hasattr(members.aq_self, 'user_foo') )
        newSecurityManager(None, acl_users.user_bar)
        mtool.createMemberArea('user_foo')
        self.failIf( hasattr(members.aq_self, 'user_foo') )
        newSecurityManager(None, acl_users.user_foo)
        mtool.setMemberareaCreationFlag()
        mtool.createMemberArea('user_foo')
        self.failIf( hasattr(members.aq_self, 'user_foo') )
        newSecurityManager(None, acl_users.all_powerful_Oz)
        mtool.setMemberareaCreationFlag()
        mtool.createMemberArea('user_foo')
        self.failUnless( hasattr(members.aq_self, 'user_foo') )

        # default content
        f = members.user_foo
        ownership = acl_users.user_foo
        localroles = ( ( 'user_foo', ('Owner',) ), )
        self.assertEqual( f.Title(), "user_foo's Home" )
        self.assertEqual( f.getOwner(), ownership )
        self.assertEqual( f.get_local_roles(), localroles,
                          'CMF Collector issue #162 (LocalRoles broken): %s'
                          % str( f.get_local_roles() ) )
        self.assertEqual( f.index_html.getOwner(), ownership,
                          'CMF Collector issue #162 (Ownership broken): %s'
                          % str( f.index_html.getOwner() ) )
        self.assertEqual( f.index_html.get_local_roles(), localroles,
                          'CMF Collector issue #162 (LocalRoles broken): %s'
                          % str( f.index_html.get_local_roles() ) )
        self.assertEqual( wtool.test_notified, f.index_html )

        # acquisition
        self.site.user_bar = 'test attribute'
        newSecurityManager(None, acl_users.user_bar)
        mtool.createMemberArea('user_bar')
        self.failUnless( hasattr(members.aq_self, 'user_bar'),
                         'CMF Collector issue #102 (acquisition bug)' )


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(MembershipToolTests),
        unittest.makeSuite(MembershipToolSecurityTests),
        ))



##parameters=
##title=Manage filter cookie
##
REQUEST=context.REQUEST
if REQUEST.get('clear_view_filter', 0):
    context.clearCookie()
    REQUEST.set('folderfilter', '')
    REQUEST.set('close_filter_form', '1')
elif REQUEST.get('set_view_filter', 0):
    filter=context.encodeFolderFilter(REQUEST)
    REQUEST.RESPONSE.setCookie('folderfilter', filter, path='/',
                               expires='Wed, 19 Feb 2020 14:28:00 GMT')
    REQUEST.set('folderfilter', '%s' % filter)



##parameters=ids, **kw
##title=Copy objects from a folder to the clipboard
##
from Products.CMFDefault.exceptions import CopyError
from Products.CMFDefault.utils import Message as _

try:
    context.manage_copyObjects(ids, context.REQUEST)
    if len(ids) == 1:
        return context.setStatus(True, _(u'Item copied.'))
    else:
        return context.setStatus(True, _(u'Items copied.'))
except CopyError:
    return context.setStatus(False, _(u'CopyError: Copy failed.'))



##parameters=
##
from Products.CMFCore.utils import getUtilityByInterfaceName
from Products.CMFDefault.utils import decode

stool = getUtilityByInterfaceName('Products.CMFCore.interfaces.ISyndicationTool')
options = {}

s_site_allowed = stool.isSiteSyndicationAllowed()
s_here_allowed = stool.isSyndicationAllowed(context)

options['title'] = context.Title()
options['description'] = context.Description()
options['s_site_allowed'] = s_site_allowed
options['s_here_allowed'] = s_here_allowed
options['s_allowed'] = s_site_allowed and s_here_allowed
options['s_tool'] = stool

return context.synPropertiesForm_template(**decode(options, script))



##parameters=
##title=Enable Syndication for a resource
##
from Products.CMFCore.utils import getUtilityByInterfaceName

REQUEST = context.REQUEST
syndtool = getUtilityByInterfaceName('Products.CMFCore.interfaces.ISyndicationTool')
syndtool.editSyInformationProperties(context,
                                     REQUEST['updatePeriod'],
                                     REQUEST['updateFrequency'],
                                     REQUEST['updateBase'],
                                     REQUEST['max_items'],
                                     REQUEST)
return REQUEST.RESPONSE.redirect(context.absolute_url() + '/synPropertiesForm?portal_status_message=Syndication+Properties+Updated.')



##############################################################################
#
# Copyright (c) 2010 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" SyndicationInfo is an adapter for IFolderish objects.
"""

from datetime import datetime

from OFS.SimpleItem import SimpleItem
from zope.annotation.interfaces import IAnnotations
from zope.component import adapts
from zope.component import getUtility
from zope.component import getAdapter
from zope.interface import alsoProvides
from zope.interface import implements
from zope.interface import Interface
from zope.interface import noLongerProvides
from zope.schema import Bool, Choice, Datetime, Int
from zope.schema.vocabulary import SimpleVocabulary

from Products.CMFCore.interfaces import IFolderish
from Products.CMFCore.interfaces import ISyndicatable
from Products.CMFCore.interfaces import ISyndicationInfo
from Products.CMFCore.interfaces import ISyndicationTool
from Products.CMFDefault.formlib.vocabulary import SimpleVocabulary
from Products.CMFDefault.SyndicationTool import SyndicationTool
from Products.CMFDefault.utils import Message as _


class SyndicationInformation(SimpleItem):
    #DEPRECATED
    """
    Existing implementation creates a full SimpleItem which is not directly
    editable
    """

    id='syndication_information'
    meta_type='SyndicationInformation'

available_periods = (
    (u'hourly', 'hourly', _(u'Hourly')),
    (u'daily', 'daily', _(u'Daily')),
    (u'weekly', 'weekly', _(u'Weekly')),
    (u'monthly', 'monthly', _(u'Monthly')),
    (u'yearly', 'yearly', _(u'Yearly')))


class ISyndicationInfo(ISyndicationInfo):

    period = Choice(
        title=_(u"Update period"),
        vocabulary=SimpleVocabulary.fromTitleItems(available_periods),
    )

    frequency = Int(
        title=_(u"Update frequency"),
        description=_(u"This is a multiple of the update period. An"
                      u" update frequency of '3' and an update period"
                      u" of 'Monthly' will mean an update every three months."),
    )

    base = Datetime(
        title=_(u"Update base"),
        description=_(u""),
        default=datetime.now()
    )

    max_items = Int(
        title=_(u"Maximum number of items"),
        description=_(u""),
        default=15
    )

    enabled = Bool(
        required=False,
    )


class SyndicationInfo(object):
    """
    Annotations adapter.
    Folders which can be syndicated are given the ISyndicatable interface
    Local syndication information is stored as a dictionary under the
    __cmf.syndication_info key of the annotations
    """

    __slots__ = ("context", "period", "frequency", "base", "max_items")
    implements(ISyndicationInfo)
    adapts(IFolderish)
    key = "__cmf.SyndicationInfo"

    def __init__(self, context):
        self.context = context

    @property
    def site_settings(self):
        """Get site syndication tool"""
        return getUtility(ISyndicationTool)

    @property
    def allowed(self):
        return self.site_settings.enabled

    def _get_property(self, attr):
        """
        Get a value from the annotation or site settings.
        """
        annotations = getAdapter(self.context, IAnnotations)
        annotation = annotations.get(self.key, None)
        if annotation is None:
            return getattr(self.site_settings, attr)
        return annotation.get(attr, getattr(self.site_settings, attr))

    def _set_property(self, attr, value):
        """
        Set a value on the annotation
        """
        annotations = getAdapter(self.context, IAnnotations)
        if not annotations.get(self.key):
            annotations[self.key] = {}
        annotation = annotations.get(self.key, None)
        annotation[attr] = value

    def get_period(self):
        return self._get_property('period')

    def set_period(self, value):
        self._set_property('period', value)

    period = property(get_period, set_period)

    def get_frequency(self):
        return self._get_property('frequency')

    def set_frequency(self, value):
        self._set_property('frequency', value)

    frequency = property(get_frequency, set_frequency)

    def get_base(self):
        return self._get_property('base')

    def set_base(self, value):
        return self._set_property('base', value)

    base = property(get_base, set_base)

    def get_max_items(self):
        return self._get_property('max_items')

    def set_max_items(self, value):
        self._set_property('max_items', value)

    max_items = property(get_max_items, set_max_items)

    @property
    def enabled(self):
        """Is syndication available for the site and a folder"""
        return self.site_settings.enabled \
               and ISyndicatable.providedBy(self.context)

    def enable(self):
        """Enable syndication for a folder"""
        alsoProvides(self.context, ISyndicatable)

    def disable(self):
        """Disable syndication for a folder"""
        self.revert()
        noLongerProvides(self.context, ISyndicatable)

    def revert(self):
        """Remove local values"""
        annotations = getAdapter(self.context, IAnnotations)
        try:
            del annotations[self.key]
        except KeyError:
            pass



##############################################################################
#
# Copyright (c) 2009 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Syndication Upgrade tests.
"""

import datetime
import unittest
from Testing import ZopeTestCase

from DateTime.DateTime import DateTime
from Products.CMFCore.PortalFolder import PortalFolder
from Products.CMFCore.interfaces import ISyndicationTool, IFolderish
from Products.CMFDefault.testing import FunctionalLayer
from Products.CMFDefault.SyndicationInfo import SyndicationInfo, ISyndicationInfo
from Products.CMFDefault.SyndicationTool import SyndicationTool
from zope.site.hooks import setSite
from zope.component import getSiteManager
from zope.component import getAdapter
from zope.testing.cleanup import cleanUp


class FunctionalUpgradeTestCase(ZopeTestCase.FunctionalTestCase):

    layer = FunctionalLayer
    _setup_fixture = 0

    def setUp(self):
        super(FunctionalUpgradeTestCase, self).setUp()
        sm = getSiteManager()
        sm.registerAdapter(SyndicationInfo, [IFolderish], ISyndicationInfo)
        syndication  = SyndicationTool()
        sm.registerUtility(syndication, ISyndicationTool)
        folder = PortalFolder("Dummy Portal Folder")
        self.folder = folder


    def _make_info(self):
        """Add an old style SyndicationInfo to the folder"""
        from Products.CMFDefault.SyndicationInfo import SyndicationInformation
        info = SyndicationInformation()
        info.syUpdateBase = DateTime()
        info.syUpdatePeriod = 1
        info.syUpdateFrequency = 1
        info.isAllowed = 1
        info.max_items = 5
        return info

    def test_upgrade(self):
        info = self._make_info()
        self.folder._setObject(info.getId(), info)
        old_info = self.folder._getOb(info.getId())
        self.assertTrue('syndication_information' in self.folder.objectIds())
        self.assertTrue(isinstance(old_info.syUpdateBase, DateTime))

        from Products.CMFDefault.upgrade.to23 import change_to_adapter
        change_to_adapter(old_info)
        self.assertFalse('syndication_information' in self.folder.objectIds())
        new_info = getAdapter(self.folder, ISyndicationInfo)
        self.assertEqual(new_info.max_items, old_info.max_items)
        self.assertEqual(new_info.period, old_info.syUpdatePeriod)
        self.assertEqual(new_info.frequency, old_info.syUpdateFrequency)
        self.assertFalse(old_info.syUpdateBase.timezoneNaive())
        self.assertTrue(new_info.base.tzinfo is None)

    def TearDown(self):
        cleanUp()


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(FunctionalUpgradeTestCase),
        ))



##############################################################################
#
# Copyright (c) 2005 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Unit tests for File module.
"""

import unittest
import Testing

from os.path import join as path_join

from App.Common import rfc1123_date
from zope.component import getSiteManager
from zope.interface.verify import verifyClass
from zope.testing.cleanup import cleanUp

from Products.CMFCore.interfaces import ICachingPolicyManager
from Products.CMFCore.testing import ConformsToContent
from Products.CMFCore.tests.base.dummy import DummyCachingManager
from Products.CMFCore.tests.base.dummy import DummyCachingManagerWithPolicy
from Products.CMFCore.tests.base.dummy import FAKE_ETAG
from Products.CMFCore.tests.base.testcase import TransactionalTest
from Products.CMFDefault import tests

TESTS_HOME = tests.__path__[0]
TEST_JPG = path_join(TESTS_HOME, 'TestImage.jpg')
TEST_SWF = path_join(TESTS_HOME, 'TestFile.swf')


class FileTests(ConformsToContent, unittest.TestCase):

    def _getTargetClass(self):
        from Products.CMFDefault.File import File

        return File

    def _makeOne(self, *args, **kw):
        return self._getTargetClass()(*args, **kw)

    def test_interfaces(self):
        from Products.CMFDefault.interfaces import IFile
        from Products.CMFDefault.interfaces import IMutableFile

        verifyClass(IFile, self._getTargetClass())
        verifyClass(IMutableFile, self._getTargetClass())

    def test_getId_on_old_File_instance(self):
        file = self._makeOne('testfile')
        self.assertEqual(file.getId(), 'testfile')
        # Mimick old instance when base classes had OFS.Image.File first
        file.__name__ = 'testfile'
        delattr(file, 'id')
        self.assertEqual(file.getId(), 'testfile')

    def test_File_setFormat(self):
        # Setting the DC.format must also set the content_type property
        file = self._makeOne('testfile', format='image/jpeg')
        self.assertEqual(file.Format(), 'image/jpeg')
        self.assertEqual(file.content_type, 'image/jpeg')
        file.setFormat('image/gif')
        self.assertEqual(file.Format(), 'image/gif')
        self.assertEqual(file.content_type, 'image/gif')

    def test_FileContentTypeUponConstruction(self):
        # Test the content type after calling the constructor with the
        # file object being passed in (http://www.zope.org/Collectors/CMF/370)
        testfile = open(TEST_JPG, 'rb')
        # Notice the cheat? File objects lack the extra intelligence that
        # picks content types from the actual file data, so it needs to be
        # helped along with a file extension...
        file = self._makeOne('testfile.jpg', file=testfile)
        testfile.close()
        self.assertEqual(file.Format(), 'image/jpeg')
        self.assertEqual(file.content_type, 'image/jpeg')


class CachingTests(TransactionalTest):

    def _getTargetClass(self):
        from Products.CMFDefault.File import File

        return File

    def _makeOne(self, *args, **kw):
        return self._getTargetClass()(*args, **kw)

    def _extractFile(self):
        f = open(TEST_SWF, 'rb')
        try:
            data = f.read()
        finally:
            f.close()

        return TEST_SWF, data

    def tearDown(self):
        cleanUp()
        TransactionalTest.tearDown(self)

    def test_index_html_with_304_from_cpm(self):
        cpm = DummyCachingManagerWithPolicy()
        getSiteManager().registerUtility(cpm, ICachingPolicyManager)
        _path, ref = self._extractFile()
        file = self._makeOne('test_file', 'test_file.swf', file=ref)
        file = file.__of__(self.app)

        mod_time = file.modified().timeTime()

        self.REQUEST.environ['IF_MODIFIED_SINCE'
                            ] = '%s;' % rfc1123_date(mod_time)
        self.REQUEST.environ['IF_NONE_MATCH'
                            ] = '%s;' % FAKE_ETAG

        data = file.index_html(self.REQUEST, self.RESPONSE)
        self.assertEqual(len(data), 0)
        self.assertEqual(self.RESPONSE.getStatus(), 304)

    def test_index_html_200_with_cpm(self):
        # should behave the same as without cpm installed
        cpm = DummyCachingManagerWithPolicy()
        getSiteManager().registerUtility(cpm, ICachingPolicyManager)
        _path, ref = self._extractFile()
        file = self._makeOne('test_file', 'test_file.swf', file=ref)
        file = file.__of__(self.app)

        mod_time = file.modified().timeTime()

        data = file.index_html(self.REQUEST, self.RESPONSE)

        self.assertEqual(len(data), len(ref))
        self.assertEqual(data, ref)
        # ICK!  'HTTPResponse.getHeader' doesn't case-flatten the key!
        self.assertEqual(self.RESPONSE.getHeader('Content-Length'.lower())
                        , str(len(ref)))
        self.assertEqual(self.RESPONSE.getHeader('Content-Type'.lower())
                        , 'application/octet-stream')
        self.assertEqual(self.RESPONSE.getHeader('Last-Modified'.lower())
                        , rfc1123_date(mod_time))

    def test_caching(self):
        large_data = '0' * 100000
        def fake_response_write(data):
            return
        response_write = self.RESPONSE.write
        self.RESPONSE.write = fake_response_write
        cpm = DummyCachingManager()
        getSiteManager().registerUtility(cpm, ICachingPolicyManager)
        original_len = len(self.RESPONSE.headers)
        obj = self._makeOne('test_file', 'test_file.swf', file=large_data)
        obj = obj.__of__(self.app)
        obj.index_html(self.REQUEST, self.RESPONSE)
        headers = self.RESPONSE.headers
        self.failUnless(len(headers) >= original_len + 3)
        self.failUnless('foo' in headers.keys())
        self.failUnless('bar' in headers.keys())
        self.assertEqual(headers['test_path'], '/test_file')
        self.RESPONSE.write = response_write

    def test_caching_policy_headers_are_canonical(self):
        """Ensure that headers set by the caching policy manager trump
        any of the same name that from time to time may be set while 
        rendering the object."""
        _path, ref = self._extractFile()

        cpm = LMDummyCachingManager()
        getSiteManager().registerUtility(cpm, ICachingPolicyManager)

        self.app.foo = self._makeOne('test_file', 'test_file.swf', file=ref)

        # index_html in OFS will set Last-Modified to ._p_mtime or current time
        self.app.foo.index_html(self.REQUEST, self.RESPONSE)

        headers = self.RESPONSE.headers
        self.assertEqual(headers['last-modified'],
                         "Sun, 06 Nov 1994 08:49:37 GMT")

# We set up a new type of dummy caching manager that sets a bogus
# last modified date.  This should be visible in the request
class LMDummyCachingManager(DummyCachingManager):

    def getHTTPCachingHeaders(self, content, view_name,
                               keywords, time=None):
        return ('Last-modified', 'Sun, 06 Nov 1994 08:49:37 GMT'),


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(FileTests),
        unittest.makeSuite(CachingTests),
        ))



##############################################################################
#
# Copyright (c) 2010 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Change user preferences.
"""

from zope.component import adapts
from zope.component import getUtility
from zope.formlib import form
from zope.interface import implements
from zope.interface import Interface
from zope.schema import Bool
from zope.schema import Choice
from zope.schema import TextLine
from zope.schema.vocabulary import SimpleVocabulary

from Products.CMFCore.interfaces import IMember
from Products.CMFCore.interfaces import IMembershipTool
from Products.CMFCore.interfaces import IPropertiesTool
from Products.CMFCore.interfaces import ISkinsTool
from Products.CMFDefault.browser.utils import memoize
from Products.CMFDefault.formlib.form import SettingsEditFormBase
from Products.CMFDefault.formlib.schema import EmailLine
from Products.CMFDefault.utils import Message as _


def portal_skins(context):
    stool = getUtility(ISkinsTool)
    return SimpleVocabulary.fromValues(stool.getSkinSelections())


class IPreferencesSchema(Interface):

    """Schema for member views.
    """

    fullname = TextLine(
        title=_(u'Full Name'),
        description=_(u'given names and surname'),
        required=False,
        missing_value=u'')

    email = EmailLine(
        title=_(u'Email Address'),
        description=_(u'info@example.org'))

    listed = Bool(
        title=_(u'Listed status'),
        description=_(u"Select to be listed on the public membership roster."))

    portal_skin = Choice(
        title=_(u'Skin'),
        vocabulary=u"cmf.AvailableSkins",
        required=False,
        missing_value='')


class PreferencesSchemaAdapter(object):

    """Adapter for IMember.
    """

    adapts(IMember)
    implements(IPreferencesSchema)

    def __init__(self, context):
        self.context = context

    def __getattr__(self, name):
        return self.context.getProperty(name)

    def __setattr__(self, name, value):
        if name in ('email', 'listed', 'portal_skin'):
            self.context.setMemberProperties({name: value})
        else:
            object.__setattr__(self, name, value)

    def _getFullName(self):
        ptool = getUtility(IPropertiesTool)
        encoding = ptool.getProperty('default_charset', None)
        return self.context.getProperty('fullname').decode(encoding)

    def _setFullName(self, value):
        ptool = getUtility(IPropertiesTool)
        encoding = ptool.getProperty('default_charset', None)
        self.context.setMemberProperties({'fullname': value.encode(encoding)})

    fullname = property(_getFullName, _setFullName)


class PreferencesFormView(SettingsEditFormBase):

    """Edit view for IPreferencesSchema.

    Only user can change his own preference.
    User can change the following preferences:
    Full name
    Email address
    Listed or unlisted
    User's chosen skin if set
    """

    label = _(u"Member Preferences")
    successMessage = _(u"Member preferences changed.")

    form_fields = form.FormFields(IPreferencesSchema)

    @memoize
    def getContent(self):
        mtool = getUtility(IMembershipTool)
        member = mtool.getAuthenticatedMember()
        return PreferencesSchemaAdapter(member)

    def applyChanges(self, data):
        changes = super(PreferencesFormView, self).applyChanges(data)
        if any('portal_skin' in v for v in changes.itervalues()):
            stool = getUtility(ISkinsTool)
            stool.updateSkinCookie()
        return changes

    def handle_change_success(self, action, data):
        self._handle_success(action, data)
        return self._setRedirect('portal_actions', 'user/preferences')

    def handle_cancel_success(self, action, data):
        return self._setRedirect('portal_actions', 'user/mystuff')



## Script (Python) "expanded_title"
##parameters=
##title=Build title which includes site title
##
from Products.CMFCore.utils import getUtilityByInterfaceName
from Products.CMFDefault.utils import decode

utool = getUtilityByInterfaceName('Products.CMFCore.interfaces.IURLTool')

site_title = utool.getPortalObject().Title()
page_title = context.Title()

if page_title != site_title:
    page_title = site_title + ": " + page_title

return decode(page_title, script)
